-------------------------------
-- RogueRotationHelper | Created by TheoChao --
-- This file handles determining the proper rotation --
-------------------------------


-- Returns the spell ID from spell name
function RogueRotationHelper:GetSpellNameById(spellId)

    if (spellId == nil) then
        return nil
    end
    
    local spellName, rank, _, _, _, _, _, _, _ = GetSpellInfo(spellId)
    return spellName
    
end

-- Create an Array of spells used within RogueRotationHelper's rotation algorithm.
RogueRotationHelper.Spells = {
    ["Garrote"]             = RogueRotationHelper:GetSpellNameById(703),
    ["Slice and Dice"]      = RogueRotationHelper:GetSpellNameById(5171),
    ["Vendetta"]            = RogueRotationHelper:GetSpellNameById(79140),
    ["Mutilate"]            = RogueRotationHelper:GetSpellNameById(1329),
    ["Backstab"]            = RogueRotationHelper:GetSpellNameById(53),
    ["Envenom"]             = RogueRotationHelper:GetSpellNameById(32645),
    ["Rupture"]             = RogueRotationHelper:GetSpellNameById(1943),
    ["Cold Blood"]          = RogueRotationHelper:GetSpellNameById(14177),
    ["Berserking"]          = RogueRotationHelper:GetSpellNameById(26297),
    ["Blood Fury"]          = RogueRotationHelper:GetSpellNameById(33697),
    ["Every Man"]           = RogueRotationHelper:GetSpellNameById(59752),
    ["Arcane Torrent"]      = RogueRotationHelper:GetSpellNameById(25046),
    ["Racial"]              = RogueRotationHelper:GetSpellNameById(nil),
    ["Tricks of the Trade"] = RogueRotationHelper:GetSpellNameById(57934),
    ["Vanish"]              = RogueRotationHelper:GetSpellNameById(1856),
    ["Overkill"]            = RogueRotationHelper:GetSpellNameById(58426),
    ["Deadly Poison"]       = RogueRotationHelper:GetSpellNameById(43581),
    ["Killing Spree"]       = RogueRotationHelper:GetSpellNameById(51690),
    ["Adrenaline Rush"]     = RogueRotationHelper:GetSpellNameById(13750),
    ["Revealing Strike"]    = RogueRotationHelper:GetSpellNameById(84617),
    ["Sinister Strike"]     = RogueRotationHelper:GetSpellNameById(1752),
    ["Eviscerate"]          = RogueRotationHelper:GetSpellNameById(2098),
    ["Blade Flurry"]        = RogueRotationHelper:GetSpellNameById(13877),
    
    ["Kick"]                = RogueRotationHelper:GetSpellNameById(1766),
    ["Feint"]               = RogueRotationHelper:GetSpellNameById(1966),
    
    ["TalentTreeSpell1"]    = RogueRotationHelper:GetSpellNameById(nil),
    ["TalentTreeSpell2"]    = RogueRotationHelper:GetSpellNameById(nil),
    
}



function RogueRotationHelper:CalculateRotation()

    local energy = UnitPower("player") + self.db.profile.EnergyOffset
    local cp = GetComboPoints("player")
    
    local murderousIntent = 0
    
    if(self.db.profile.SuggestBackstab == true) and (UnitHealth("target") <= UnitHealthMax("target") * 0.35) then
        murderousIntent = 1
    end

    local spell = ""
    local nextspell = ""
    
    local garrote = 0
    local slice_and_dice = 0
    local rupture = 0
    local vendetta = 0
    local ruptureLevel = 0
    local talent1 = 0
    local talent2 = 0
    local tott = 0
    local feint = 0
    local kick = 0
    local deadly_poison_stack_count = 0
    local deadly_poison = 0
    local vanish = 0
    local overkill = 0
    local coldblood = 0
    local racial = 0
    local envenom = 0
    local talent_tree = RogueRotationHelper.db.profile.TalentTree
    
    local soundfile = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\pingit.mp3"

    -- get the time left for various buffs/debuffs
    -- Slice and Dice
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitBuff("player", self.Spells["Slice and Dice"])
    if name ~= nil then
        slice_and_dice = expirationTime - GetTime()
    end
    
    -- Overkill
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitBuff("player", self.Spells["Overkill"])
    if name ~= nil then
        overkill = expirationTime - GetTime()
    end

     -- Feint
    local start, duration, enabled = GetSpellCooldown(self.Spells["Feint"]);
    if duration ~= nil then 
        feint = duration + start - GetTime()
    end
        
     -- Tricks
    local start, duration, enabled = GetSpellCooldown(self.Spells["Tricks of the Trade"]);
    if duration ~= nil then 
        tott = duration + start - GetTime()
    end
        
    -- Ruture
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitDebuff("target", self.Spells["Rupture"])
    if name ~= nil and isMine == "player" then
        rupture = expirationTime - GetTime()
        ruptureLevel = floor(duration) / 2 - 5
    end

    -- Vendetta
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitDebuff("target", self.Spells["Vendetta"])
    if name ~= nil and isMine == "player" then
        vendetta = expirationTime - GetTime()
    end
    
    -- Cold Blood
    local start, duration, enabled = GetSpellCooldown(self.Spells["Cold Blood"]);
    if duration ~= nil then 
        coldblood = duration + start - GetTime()
    end

    -- Deadly Poison
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitDebuff("target", self.Spells["Deadly Poison"])
    if name ~= nil and isMine == "player" then
        deadly_poison_stack_count = count
        deadly_poison = expirationTime - GetTime()
    end
    
    -- Envenom
    local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitBuff("player", self.Spells["Envenom"]);
    if name ~= nil then
        envenom = expirationTime - GetTime()
    end
    

    -- Rotation suggestion algorithm...
    -- first determine if the talent tree is assination or combat
    if talent_tree == 1 then
        -- Stealth active
        if UnitAura("player", "Stealth") ~= nil then
            -- Open with Garrote
            if (energy >= 50) then
                spell = self.Spells["Garrote"]
            else
                nextspell = self.Spells["Garrote"];
            end
       -- Initial rupture
        elseif (cp >= 1) and (rupture == 0.0) then
            -- Check required energy
            if (energy >= 25) then
                spell = self.Spells["Rupture"];
            else
                nextspell = self.Spells["Rupture"];
            end
            
        -- Slice and Dice 
        elseif (cp >= 1) and (slice_and_dice == 0.0) then
            -- Check required energy
            if (energy >= 25) then
                spell = self.Spells["Slice and Dice"];
            else
                nextspell = self.Spells["Slice and Dice"];
            end
            
        -- Finish to refresh Slice and Dice if low on timer
        elseif (cp >= 1) and (deadly_poison_stack_count >= 1) and (slice_and_dice <= self.db.profile.SliceTimer and slice_and_dice > 0) then
            -- Check required energy
            if (energy >= 35) then
                spell = self.Spells["Envenom"];
            else
                nextspell = self.Spells["Envenom"];
            end
               
        -- Rupture if low on timer (but don't cap out on energy
        elseif (rupture <= self.db.profile.RuptureTimer) and (cp >= 1 ) and (energy < 105 or energy >= 120) then 
            -- Queue if not quite finished, check required energy, make sure that able to refresh (or it's close to expiring) and don't go too early
            if (energy >= 25 and (cp >= ruptureLevel or rupture <.5) and rupture < 2) then
                spell = self.Spells["Rupture"];
            else
                -- if there's some time and lower than the rupture point - build more cp
                if (rupture>2.5 and cp < ruptureLevel and energy>60) then
                    if(murderousIntent == 1 and cp < 5) then
                        spell = self.Spells["Backstab"];
                    elseif(murderousIntent == 0 and cp < 4) then
                        spell = self.Spells["Mutilate"];
                    else
                        nextspell = self.Spells["Rupture"]
                    end
                else
                    nextspell = self.Spells["Rupture"];
                end
            end
        
         -- Cold Blood
        elseif (coldblood <= 0 and ((cp >= 5 and vendetta > 0) or (cp >= 4 and vendetta > 0 and vendetta < 10))) then
            spell = self.Spells["Cold Blood"];
        
        -- Suggest Envenom (in a single case to account for murderous intent taking to 5 cp)
        elseif ( cp >= 4 + murderousIntent) then
            --make sure that energy is pooled, there's dp up, and envenom isn't already on (unless maxing out on energy)
            if ( energy >= 55 and deadly_poison_stack_count == 5 and (envenom == 0 or energy >= 105)) then
                spell = self.Spells["Envenom"];
            else
                -- or if there's no stacks of dp, assume it's not on and eviscerate
                if (deadly_poison_stack_count == 0) then
                        if (energy >= 35) then
                            spell = self.Spells["Eviscerate"];
                        else
                            nextspell = self.Spells["Eviscerate"];
                        end
                else
                    nextspell = self.Spells["Envenom"];
                end
            end
            
        -- build up combo points
        else
            if(murderousIntent == 1 ) then
                if (energy >= 60) then
                    spell = self.Spells["Backstab"];
                else
                    nextspell = self.Spells["Backstab"];
                end
            
            else
                if (energy >= 55) then
                    spell = self.Spells["Mutilate"];
                else
                    nextspell = self.Spells["Mutilate"];
                end
                
            end
        end
    -- in combat spec
    else
        -- Stealth active
        if UnitAura("player", "Stealth") ~= nil then
            -- Open with Garrote
            if (energy >= 50) then
                spell = self.Spells["Garrote"]
            else
                nextspell = self.Spells["Garrote"];
            end

        -- Slice and Dice if running low
        elseif (cp >= 1 and cp <4) and (slice_and_dice <= self.db.profile.SliceTimer) then
            -- Check required energy
            if (energy >= 25) then
                spell = self.Spells["Slice and Dice"];
            else
                nextspell = self.Spells["Slice and Dice"];
            end
            
        -- Rupture if low on timer/out
        elseif ( cp>=5 and rupture <= self.db.profile.RuptureTimer and false ) then 
            -- Queue if not quite finished, check required energy
            if (energy >= 25) then
                spell = self.Spells["Rupture"];
            else
                nextspell = self.Spells["Rupture"];
            end
            
        -- Suggest Eviscerate
        elseif (cp >= 5) then
            -- Check required energy
            if (energy >= 35) then
                spell = self.Spells["Eviscerate"];
            else
                nextspell = self.Spells["Eviscerate"];
            end
            
        -- build cp
        else
            -- if at 4, rs for the single point
            if( cp >= 4 ) then
                if (energy >= 40) then
                    spell = self.Spells["Revealing Strike"];
                else
                    nextspell = self.Spells["Revealing Strike"];
                end
            
            -- otherwise get 1 or 2 with ss
            else
                if (energy >= 39) then
                    spell = self.Spells["Sinister Strike"];
                else
                    nextspell = self.Spells["Sinister Strike"];
                end
            end
        end

    end
    
    if spell == "" then
        if self.db.profile.SuggestTricks and tott < 1 then
            spell = self.Spells["Tricks of the Trade"];
        elseif self.db.profile.SuggestFeint and feint < 1 then
            spell = self.Spells["Feint"];
        else
        end
    end
                        
    soundfile = self.db.profile.SpellSounds[spell]
    
    self.displayFrame_current.Texture:SetTexture(GetSpellTexture(spell))
    self.displayFrame_next.Texture:SetTexture(GetSpellTexture(nextspell))
    
    -- check for blade flurry
    if( talent_tree==2 and UnitBuff("player", self.Spells["Blade Flurry"]) ) then
        self.displayFrame_current.Border:SetTexture(1,0,0,1)
    else
        self.displayFrame_current.Border:SetTexture(0,0,0,0)
    end
        
    -- global cooldowns
    if (spell ~= "") and (spell ~= nil) then
        local start, dur = GetSpellCooldown(spell)
        if dur ~= 0 or start ~= 0 then
            -- Display global cooldowns
            self.displayFrame_current.Frame:SetCooldown(start, dur)
            
            -- Doesn't display anything as of yet, may be used in future with better queueing algorithm
            if(nextspell ~= "") and (nextspell ~= nil) then
                self.displayFrame_next.Frame:SetCooldown(start, dur)
            end
            self.db.profile.lastSpell = ""
        else
            if(self.db.profile.lastSpell ~= spell) then
                if self.db.profile.SoundEnabled then
                    PlaySoundFile(soundfile, "MASTER")
                end
                self.db.profile.lastSpell = spell
            end
        end
    end
end
